@@ -1,5 +1,5 @@
 ;;; handle the posix functions
-;;; Sam Steingold 1999-2007
+;;; Sam Steingold 1999-2008
 
 (defpackage #:posix
   (:use #:common-lisp #:ext)
@@ -11,11 +11,12 @@
    #:stream-lock #:with-stream-lock #:duplicate-handle #:copy-file
    #:file-owner #:physical-memory #:stream-options #+unix #:string-time
    #+(or :win32 :cygwin) #:file-properties #+unix #:make-xterm-io-stream
-   #:priority #:process-id #:openlog #:setlogmask #:syslog #:closelog
-   #:getpgid #:setpgrp #:getsid #:setsid #:setpgid #:kill #:sync
-   #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:gamma #:lgamma #:ffs))
+   #:priority #:openlog #:setlogmask #:syslog #:closelog #:process-id #:getppid
+   #:getsid #:setsid #:getpgrp #:setpgrp #:setreuid #:setregid #:kill #:sync
+   #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:tgamma #:lgamma #:ffs))
 
 (pushnew :syscalls *features*)
+(provide "syscalls")
 (in-package #:posix)
 
 (setf (documentation (find-package '#:posix) 'sys::impnotes) "syscalls")
@@ -28,11 +29,13 @@
 )
 ;;; ============================================================
 #+unix (progn
-(export '(getuid getgid geteuid getegid))
+(export '(getuid getgid geteuid getegid getpgid getgroups))
 (defsetf getuid posix::%setuid)
 (defsetf getgid posix::%setgid)
 (defsetf geteuid posix::%seteuid)
 (defsetf getegid posix::%setegid)
+(defsetf getpgid posix::%setpgid)
+(defsetf getgroups posix::%setgroups)
 )
 ;;; ============================================================
 (defmacro with-stream-lock ((stream &rest options) &body body)
@@ -49,7 +52,7 @@
   `(%syslog ,severity ,facility (format nil ,format ,@args)))
 ;;; ============================================================
 (defsetf priority (pid &optional which) (value)
-  `(set-priority ,value ,pid ,which))
+  `(%set-priority ,value ,pid ,which))
 ;;; ============================================================
 (defstruct (hostent (:constructor
                      make-hostent (name aliases addr-list addrtype)))
@@ -358,7 +361,7 @@
   (let* ((tmps (mkstemp "/tmp/clisp-x-io-XXXXXX"))
          (pipe (namestring tmps))
          xio
-         (clos::*warn-if-gf-already-called* nil))
+         (clos::*enable-clos-warnings* nil))
     (close tmps) (delete-file tmps)
     (mknod tmps :FIFO :RWXU)
     (setq title (string title))
@@ -380,11 +383,14 @@
       (with-open-file (s pipe :direction :output)
         (write-line (SYS::TEXT "Bye.") s))
       (delete-file pipe)
-      (let ((clos::*warn-if-gf-already-called* nil))
+      (let ((clos::*enable-clos-warnings* nil))
         (remove-method #'close (find-method #'close '(:after) `((eql ,xio))))))
     xio))
 
 ;;;--------------------------------------------------------------------------
+#+FFI (export '(fopen fdopen fclose fflush ; fgetc fputc ungetc
+                clearerr feof ferror fileno))
+;;;--------------------------------------------------------------------------
 (defun ffs (n) (integer-length (logand n (- n))))
 ;; http://www.opengroup.org/onlinepubs/009695399/functions/ffs.html
 ;; this lisp implementation is about 10% faster than using FFI:
