@@ -1,6 +1,35 @@
 ;; run the test suit:
 
+;;; --- helpers for misc tests ---
 (defun princ-error (c) (format t "~&[~A]: ~A~%" (type-of c) c))
+(defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
+  "Print the object on its own line and return it. Used in many tests!"
+  (fresh-line) (prin1 object) (terpri) object)
+(defun kill-down (name)
+  (dolist (f (directory (ext:string-concat name "**")))
+    (format t "~&removing ~S~%" f)
+    (if (pathname-name f)
+        (delete-file f)
+        (ext:delete-dir f))))
+(defun rmrf (name)
+  (ext:dir (ext:string-concat name "**"))
+  (kill-down name)
+  (format t "~&removing ~S~%" name)
+  (ext:delete-dir name))
+(defun prepare-dir (name)
+  (ensure-directories-exist name :verbose t)
+  (kill-down name))
+(defun show-file (file)         ; return line count
+  (with-open-file (st file :direction :input)
+    (format t "~&~S: ~:D byte~:P:~%" file (file-length st))
+    (loop :for l = (read-line st nil nil) :while l :count t
+      :do (format t "--> ~S~%" l))))
+(defun finish-file (file) (prog1 (show-file file) (delete-file file)))
+(defun post-compile-file-cleanup (file)
+  (delete-file file)
+  (delete-file (compile-file-pathname file))
+  #+clisp (delete-file (make-pathname :type "lib" :defaults file)))
+;;; end helpers
 
 #+OLD-CLISP
 ;; Binding *ERROR-HANDLER* is a hammer technique for catching errors. It also
@@ -73,6 +102,22 @@
                (error "eval: ~S; compile: ~S" e-value c-value))
              e-value))))
 
+;; compare 2 sequences:
+;; list of (pos first second);
+;; last 3 elements can be smaller length, tail1, tail2
+(defun diff-seq (s1 s2 &key (test #'eql))
+  (let ((pos 0))
+    (nconc
+     (delete nil
+             (map 'list (lambda (e1 e2)
+                          (prog1 (unless (funcall test e1 e2)
+                                   (list pos e1 e2))
+                            (incf pos)))
+                  s1 s2))
+     (let ((l1 (length s1)) (l2 (length s2)))
+       (cond ((< l1 l2) (list l1 nil (subseq s2 l1)))
+             ((< l2 l1) (list l2 (subseq s1 l2) nil)))))))
+
 (defgeneric pretty-compare (result my-result log)
   (:documentation "print a pretty comparison of two results")
   (:method ((result sequence) (my-result sequence) (log stream))
@@ -103,10 +148,6 @@
                 slot s-r lisp-implementation s-m (equal s-r s-m)))))
   (:method ((result t) (my-result t) (log stream)))) ; do nothing
 
-(defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
-  "Print the object on its own line and return it. Used in many tests!"
-  (fresh-line) (prin1 object) (terpri) object)
-
 (defun type-error-handler (err)
   "Print the condition and THROW.
 Usage: (handler-bind ((type-error #'type-error-handler)) ...)"
@@ -149,7 +190,8 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                              my-result error-message)
                  (pretty-compare result my-result log)
                  (format log "~[~*~:;OUT:~%~S~%~]~[~*~:;ERR:~%~S~]~2%"
-                         (length out) out (length err) err))))))
+                         (length out) out (length err) err)
+                 (force-output log))))))
     (values total-count error-count)))
 
 (defmacro check-ignore-errors (&body body)
@@ -201,7 +243,8 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                    (format log "~&Form: ~S~%CORRECT: ~S~%~7A: ~S~%~
                                 ~[~*~:;OUT:~%~S~%~]~[~*~:;ERR:~%~S~]~2%"
                                form errtype lisp-implementation my-result
-                               (length out) out (length err) err)))))))
+                               (length out) out (length err) err)
+                   (force-output log)))))))
     (values total-count error-count)))
 
 (defvar *run-test-tester* #'do-test)
@@ -244,10 +287,10 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
   "res = list of RUN-TEST return values (testname total-count error-count)"
   (let ((error-count (reduce #'+ res :key #'third)))
     (format
-     t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+     t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
      (length res) error-count (reduce #'+ res :key #'second))
     (loop :with here = (truename "./") :for rec :in res :for count :upfrom 1 :do
-      (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+      (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
               count (enough-namestring (first rec) here)
               (third rec) (second rec)))
     error-count))
@@ -277,6 +320,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                   #+CLISP           "defhash"
                   #+(and CLISP UNICODE) "encoding"
                                     "eval20"
+                  #+CLISP           "ext-clisp"
                   #+(and CLISP FFI) "ffi"
                                     "floeps"
                   #-OpenMCL         "format"
@@ -314,6 +358,8 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                   #+XCL             "tprint"
                   #+XCL             "tread"
                                     "type"
+                  #-(or)            "unportable"
+                  #+(and CLISP mt)  "mt"
                   #+CLISP           "weak"
                   #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) "weakhash"
                   #+(or CLISP LISPWORKS) "weakhash2"))
