@@ -7,12 +7,15 @@
 find . -type f -follow -perm +111 '(' \
      -name '*.in' \
   -o -name '*.xml' \
+  -o -name '*.xsl' \
+  -o -name '*.css' \
   -o -name '*.html' \
   -o -name '*.png' \
   -o -name '*.xsl' \
   -o -name '*.m4' \
   -o -name '*.h' \
   -o -name '*.c' \
+  -o -name '*.d' \
   -o -name '*.lisp' \
   -o -name '*.tst' \
   -o -name '*.bat' \
@@ -23,4 +26,4 @@ find . -type f -follow -perm +111 '(' \
   ')' \
   -exec chmod -c a-x '{}' ';'
 
-find . -name configure -exec chmod -c +x '{}' ';'
+find . '(' -name configure -o -name autogen.sh ')' -exec chmod -c +x '{}' ';'
